package com.sun.corba.se.impl.io;

import com.sun.corba.se.spi.orb.ORBVersion;
import com.sun.corba.se.spi.orb.ORBVersionFactory;
import com.sun.corba.se.spi.orb.ORB;
import org.omg.CORBA.portable.ValueInputStream;
import java.io.ObjectStreamClass;
import org.omg.CORBA_2_3.portable.InputStream;
import java.util.HashMap;
import java.io.StreamCorruptedException;
import java.io.InvalidClassException;
import java.util.Map;
import java.io.NotActiveException;
import java.io.IOException;
import com.sun.corba.se.impl.logging.UtilSystemException;
import com.sun.corba.se.impl.logging.OMGSystemException;
import java.io.ObjectInputStream;

public abstract class InputStreamHook extends ObjectInputStream
{
    static final OMGSystemException omgWrapper;
    static final UtilSystemException utilWrapper;
    protected InputStreamHook.ReadObjectState readObjectState;
    protected static final InputStreamHook.ReadObjectState DEFAULT_STATE;
    protected static final InputStreamHook.ReadObjectState IN_READ_OBJECT_OPT_DATA;
    protected static final InputStreamHook.ReadObjectState IN_READ_OBJECT_NO_MORE_OPT_DATA;
    protected static final InputStreamHook.ReadObjectState IN_READ_OBJECT_DEFAULTS_SENT;
    protected static final InputStreamHook.ReadObjectState NO_READ_OBJECT_DEFAULTS_SENT;
    protected static final InputStreamHook.ReadObjectState IN_READ_OBJECT_REMOTE_NOT_CUSTOM_MARSHALED;
    protected static final InputStreamHook.ReadObjectState IN_READ_OBJECT_PAST_DEFAULTS_REMOTE_NOT_CUSTOM;
    
    public InputStreamHook() throws IOException {
        this.readObjectState = InputStreamHook.DEFAULT_STATE;
    }
    
    @Override
    public void defaultReadObject() throws IOException, ClassNotFoundException, NotActiveException {
        this.readObjectState.beginDefaultReadObject(this);
        this.defaultReadObjectDelegate();
        this.readObjectState.endDefaultReadObject(this);
    }
    
    abstract void defaultReadObjectDelegate();
    
    abstract void readFields(Map p0) throws InvalidClassException, StreamCorruptedException, ClassNotFoundException, IOException;
    
    @Override
    public ObjectInputStream.GetField readFields() throws IOException, ClassNotFoundException, NotActiveException {
        HashMap<Object, Object> hashMap = new HashMap();
        this.readFields(hashMap);
        this.readObjectState.endDefaultReadObject(this);
        return new InputStreamHook.HookGetFields(hashMap);
    }
    
    protected void setState(InputStreamHook.ReadObjectState readObjectState) {
        this.readObjectState = readObjectState;
    }
    
    protected abstract byte getStreamFormatVersion();
    
    abstract org.omg.CORBA_2_3.portable.InputStream getOrbStream();
    
    protected void throwOptionalDataIncompatibleException() {
        throw InputStreamHook.omgWrapper.rmiiiopOptionalDataIncompatible2();
    }
    
    static {
        omgWrapper = OMGSystemException.get("rpc.encoding");
        utilWrapper = UtilSystemException.get("rpc.encoding");
        DEFAULT_STATE = new InputStreamHook.DefaultState();
        IN_READ_OBJECT_OPT_DATA = new InputStreamHook.InReadObjectOptionalDataState();
        IN_READ_OBJECT_NO_MORE_OPT_DATA = new InputStreamHook.InReadObjectNoMoreOptionalDataState();
        IN_READ_OBJECT_DEFAULTS_SENT = new InputStreamHook.InReadObjectDefaultsSentState();
        NO_READ_OBJECT_DEFAULTS_SENT = new InputStreamHook.NoReadObjectDefaultsSentState();
        IN_READ_OBJECT_REMOTE_NOT_CUSTOM_MARSHALED = new InputStreamHook.InReadObjectRemoteDidNotUseWriteObjectState();
        IN_READ_OBJECT_PAST_DEFAULTS_REMOTE_NOT_CUSTOM = new InputStreamHook.InReadObjectPastDefaultsRemoteDidNotUseWOState();
    }
    
    private class HookGetFields extends ObjectInputStream.GetField
    {
        private Map fields;
        
        HookGetFields(Map fields) {
            this.fields = null;
            this.fields = fields;
        }
        
        @Override
        public ObjectStreamClass getObjectStreamClass() {
            return null;
        }
        
        @Override
        public boolean defaulted(String s) throws IOException, IllegalArgumentException {
            return !this.fields.containsKey(s);
        }
        
        @Override
        public boolean get(String s, boolean b) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return b;
            }
            return (boolean)this.fields.get(s);
        }
        
        @Override
        public char get(String s, char c) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return c;
            }
            return (char)this.fields.get(s);
        }
        
        @Override
        public byte get(String s, byte b) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return b;
            }
            return (byte)this.fields.get(s);
        }
        
        @Override
        public short get(String s, short n) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return n;
            }
            return (short)this.fields.get(s);
        }
        
        @Override
        public int get(String s, int n) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return n;
            }
            return (int)this.fields.get(s);
        }
        
        @Override
        public long get(String s, long n) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return n;
            }
            return (long)this.fields.get(s);
        }
        
        @Override
        public float get(String s, float n) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return n;
            }
            return (float)this.fields.get(s);
        }
        
        @Override
        public double get(String s, double n) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return n;
            }
            return (double)this.fields.get(s);
        }
        
        @Override
        public Object get(String s, Object o) throws IOException, IllegalArgumentException {
            if (this.defaulted(s)) {
                return o;
            }
            return this.fields.get(s);
        }
        
        @Override
        public String toString() {
            return this.fields.toString();
        }
    }
    
    protected static class ReadObjectState
    {
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) throws IOException {
        }
        
        public void endUnmarshalCustomValue(InputStreamHook inputStreamHook) throws IOException {
        }
        
        public void beginDefaultReadObject(InputStreamHook inputStreamHook) throws IOException {
        }
        
        public void endDefaultReadObject(InputStreamHook inputStreamHook) throws IOException {
        }
        
        public void readData(InputStreamHook inputStreamHook) throws IOException {
        }
    }
    
    protected static class DefaultState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) throws IOException {
            if (b2) {
                if (b) {
                    inputStreamHook.setState(InputStreamHook.IN_READ_OBJECT_DEFAULTS_SENT);
                }
                else {
                    try {
                        if (inputStreamHook.getStreamFormatVersion() == 2) {
                            ((ValueInputStream)inputStreamHook.getOrbStream()).start_value();
                        }
                    }
                    catch (Exception ex) {}
                    inputStreamHook.setState(InputStreamHook.IN_READ_OBJECT_OPT_DATA);
                }
            }
            else if (b) {
                inputStreamHook.setState(InputStreamHook.NO_READ_OBJECT_DEFAULTS_SENT);
            }
            else {
                throw new StreamCorruptedException("No default data sent");
            }
        }
    }
    
    protected static class InReadObjectRemoteDidNotUseWriteObjectState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) {
            throw InputStreamHook.utilWrapper.badBeginUnmarshalCustomValue();
        }
        
        @Override
        public void endDefaultReadObject(InputStreamHook inputStreamHook) {
            inputStreamHook.setState(InputStreamHook.IN_READ_OBJECT_PAST_DEFAULTS_REMOTE_NOT_CUSTOM);
        }
        
        @Override
        public void readData(InputStreamHook inputStreamHook) {
            inputStreamHook.throwOptionalDataIncompatibleException();
        }
    }
    
    protected static class InReadObjectPastDefaultsRemoteDidNotUseWOState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) {
            throw InputStreamHook.utilWrapper.badBeginUnmarshalCustomValue();
        }
        
        @Override
        public void beginDefaultReadObject(InputStreamHook inputStreamHook) throws IOException {
            throw new StreamCorruptedException("Default data already read");
        }
        
        @Override
        public void readData(InputStreamHook inputStreamHook) {
            inputStreamHook.throwOptionalDataIncompatibleException();
        }
    }
    
    protected static class InReadObjectDefaultsSentState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) {
            throw InputStreamHook.utilWrapper.badBeginUnmarshalCustomValue();
        }
        
        @Override
        public void endUnmarshalCustomValue(InputStreamHook inputStreamHook) {
            if (inputStreamHook.getStreamFormatVersion() == 2) {
                ((ValueInputStream)inputStreamHook.getOrbStream()).start_value();
                ((ValueInputStream)inputStreamHook.getOrbStream()).end_value();
            }
            inputStreamHook.setState(InputStreamHook.DEFAULT_STATE);
        }
        
        @Override
        public void endDefaultReadObject(InputStreamHook inputStreamHook) throws IOException {
            if (inputStreamHook.getStreamFormatVersion() == 2) {
                ((ValueInputStream)inputStreamHook.getOrbStream()).start_value();
            }
            inputStreamHook.setState(InputStreamHook.IN_READ_OBJECT_OPT_DATA);
        }
        
        @Override
        public void readData(InputStreamHook inputStreamHook) throws IOException {
            org.omg.CORBA.ORB orb = inputStreamHook.getOrbStream().orb();
            if (orb == null || !(orb instanceof ORB)) {
                throw new StreamCorruptedException("Default data must be read first");
            }
            ORBVersion orbVersion = ((ORB)orb).getORBVersion();
            if (ORBVersionFactory.getPEORB().compareTo(orbVersion) <= 0 || orbVersion.equals(ORBVersionFactory.getFOREIGN())) {
                throw new StreamCorruptedException("Default data must be read first");
            }
            return;
        }
    }
    
    protected static class InReadObjectOptionalDataState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void beginUnmarshalCustomValue(InputStreamHook inputStreamHook, boolean b, boolean b2) {
            throw InputStreamHook.utilWrapper.badBeginUnmarshalCustomValue();
        }
        
        @Override
        public void endUnmarshalCustomValue(InputStreamHook inputStreamHook) throws IOException {
            if (inputStreamHook.getStreamFormatVersion() == 2) {
                ((ValueInputStream)inputStreamHook.getOrbStream()).end_value();
            }
            inputStreamHook.setState(InputStreamHook.DEFAULT_STATE);
        }
        
        @Override
        public void beginDefaultReadObject(InputStreamHook inputStreamHook) throws IOException {
            throw new StreamCorruptedException("Default data not sent or already read/passed");
        }
    }
    
    protected static class InReadObjectNoMoreOptionalDataState extends InputStreamHook.InReadObjectOptionalDataState
    {
        @Override
        public void readData(InputStreamHook inputStreamHook) throws IOException {
            inputStreamHook.throwOptionalDataIncompatibleException();
        }
    }
    
    protected static class NoReadObjectDefaultsSentState extends InputStreamHook.ReadObjectState
    {
        @Override
        public void endUnmarshalCustomValue(InputStreamHook inputStreamHook) throws IOException {
            if (inputStreamHook.getStreamFormatVersion() == 2) {
                ((ValueInputStream)inputStreamHook.getOrbStream()).start_value();
                ((ValueInputStream)inputStreamHook.getOrbStream()).end_value();
            }
            inputStreamHook.setState(InputStreamHook.DEFAULT_STATE);
        }
    }
}
